options linesize = 256 pagesize = 32000 orientation = landscape spool symbolgen;

filename cadet './ins/cadets.csv';
filename goal './ins/goal.txt';
/* read in the cadet data */
data cadets_loaded;
  infile cadet DLM=',' DSD MISSOVER;
  input OMS $ SEX $ RACE $ prevbr $ BR1 $ BR2 $ BR3 $ BR4 $ BR5 $ acd $ ; 
  if _N_ > 1;
  omsnbr = input(OMS,9.);
  ac = input(acd,9.);
  RCE='1';
  if RACE ne '1' then RCE='2';
run;

proc sort data=cadets_loaded;
   by descending omsnbr;

data cadets;
  set cadets_loaded;
  rank=_N_;
run;

/* this reads in the demand data goals.txt */
data goalst(drop=goal);
  infile goal missover;
  input BR $1-2 goal $ ;
  goalnbr = input(goal,9.);
run;

proc sql; select count(*) as cnt into :all_cnt from cadets; quit;
proc sql; select count(*) into :fem_cnt from cadets where SEX='F'; quit;
proc sql; select count(*) into :a1_cnt from cadets where ac = 1 ; quit;
proc sql; select count(*) into :a2_cnt from cadets where ac = 2 ; quit;
proc sql; select count(*) into :a3_cnt from cadets where ac = 3 ; quit;
proc sql; select count(*) into :a4_cnt from cadets where ac = 4 ; quit;
proc sql; select count(*) into :min_cnt from cadets where RACE ne '1' ; quit;
proc sql; select count(*) into :min2_cnt from cadets where RACE eq '2' ; quit;
proc sql; select count(*) into :min3_cnt from cadets where RACE eq '3' ; quit;
proc sql; select count(*) into :min4_cnt from cadets where RACE eq '4' ; quit;
proc sql; select count(*) into :min5_cnt from cadets where RACE eq '5' ; quit;
proc sql; select avg(omsnbr) into :avg_oms from cadets; quit;
proc sql; select sum(goalnbr) as cnt into :ca_cnt from goalst where BR in ('IN','AR','FA'); quit;

data goals;
  set goalst;
  femgoals=round(goalnbr * &fem_cnt/(&all_cnt-&ca_cnt),1);
  if BR in ('IN' 'AR' 'FA') then femgoals = 0;
  mingoals=round(goalnbr * &min_cnt/&all_cnt,1);
  min2goals=round(goalnbr * &min2_cnt/&all_cnt,1);
  min3goals=round(goalnbr * &min3_cnt/&all_cnt,1);
  min4goals=round(goalnbr * &min4_cnt/&all_cnt,1);
  min5goals=round(goalnbr * &min5_cnt/&all_cnt,1);
  a1goals=round(goalnbr * &a1_cnt/&all_cnt,1);
  a2goals=round(goalnbr * &a2_cnt/&all_cnt,1);
  a3goals=round(goalnbr * &a3_cnt/&all_cnt,1);
  a4goals=round(goalnbr * &a4_cnt/&all_cnt,1);
  qgoals=0.998*goalnbr*&avg_oms;
run;

proc optmodel printlevel=2;

  set C;
  string OMS{C};
  string SEX{C};
  string RACE{C};
  string prevbr{C};
  string BR1{C};
  string BR2{C};
  string BR3{C};
  string BR4{C};
  string BR5{C};
  number omsnbr{C};
  number ac{C};
  string RCE{C};
  number rank{C};
  read data cadets into C=[_N_]OMS SEX RACE prevbr BR1 BR2 BR3 BR4 BR5 omsnbr ac RCE rank;

  print OMS SEX RACE prevbr BR1 BR2 BR3 BR4 BR5 omsnbr ac RCE rank; 

  set B;
  string BR{B};
  number goalnbr{B};
  number femgoals{B};
  number mingoals{B};
  number min2goals{B};
  number min3goals{B};
  number min4goals{B};
  number min5goals{B};
  number a1goals{B};
  number a2goals{B};
  number a3goals{B};
  number a4goals{B};
  number qgoals{B};
  read data goals into B=[_N_] BR goalnbr femgoals mingoals 
     min4goals min3goals min2goals min5goals a1goals a2goals a3goals a4goals qgoals;

  print BR goalnbr femgoals mingoals min4goals min3goals min2goals min5goals a1goals a2goals a3goals a4goals qgoals;

  number arc{B,C};

  for{i in B, j in C}
  if BR[i]=BR1[j] then arc[i,j] = 6/rank[j];
  else if BR[i]=BR2[j] then arc[i,j] = 5/rank[j];
  else if BR[i]=BR3[j] then arc[i,j] = 4/rank[j];
  else if BR[i]=BR4[j] then arc[i,j] = 3/rank[j];
  else if BR[i]=BR5[j] then arc[i,j] = 2/rank[j];
  else arc[i,j]=1/rank[j];

/*  print arc;  */


  set F = setof{i in C: SEX[i]='F'}<i>;

  set A1 = setof{i in C: AC[i]=1}<i>;
  set A2 = setof{i in C: AC[i]=2}<i>;
  set A3 = setof{i in C: AC[i]=3}<i>;
  set A4 = setof{i in C: AC[i]=4}<i>;
  set R2 = setof{i in C: RACE[i]='2'}<i>;
  set R3 = setof{i in C: RACE[i]='3'}<i>;
  set R4 = setof{i in C: RACE[i]='4'}<i>;
  set R5 = setof{i in C: RACE[i]='5'}<i>;
  set M = setof{i in C: RCE[i]='2'}<i>;

  set CA = setof{i in B: BR[i] eq 'IN' OR BR[i] eq 'AR' OR BR[i] eq 'FA'}<i>;
  set NCA = setof{i in B: BR[i] ne 'IN' AND BR[i] ne 'AR' AND BR[i] ne 'FA'}<i>;
  print{i in CA} BR[i];

  print{i in NCA} BR[i];

  var x{B,C} >= 0; 

  maximize total_score = sum{i in B, j in C}arc[i,j]*x[i,j];
  constraint supply{j in C}: sum{i in B}x[i,j]=1;
  constraint demand{i in B}: sum{j in C}x[i,j]>=goalnbr[i];  
  constraint fems_no_ca{i in CA}: sum{j in F}x[i,j]=0;

/*constraint fcadem{i in B}: sum{j in F}x[i,j]>=femgoals[i]; 
  constraint quality{i in B}: sum{j in C}omsnbr[j]*x[i,j]>=qgoals[i];  
  constraint mindem{i in B}: sum{j in M}x[i,j]>=mingoals[i]; 
  constraint min2dem{i in B}: sum{j in R2}x[i,j]>=min2goals[i]; 
  constraint min3dem{i in B}: sum{j in R3}x[i,j]>=min3goals[i];  
  constraint min4dem{i in B}: sum{j in R4}x[i,j]>=min4goals[i];  
  constraint min5dem{i in B}: sum{j in R5}x[i,j]>=min5goals[i];
  constraint a1type{i in B}: sum{j in A1}x[i,j]>=a1goals[i];  
  constraint a2type{i in B}: sum{j in A2}x[i,j]>=a2goals[i];  
  constraint a3type{i in B}: sum{j in A3}x[i,j]>=a3goals[i];  
  constraint a4type{i in B}: sum{j in A4}x[i,j]>=a4goals[i];*/
  
  solve;
/*  
  print x;  
*/
  create data solt from [B C]={i in B, j in C: x[i,j]>0} BR[i] rank[j] arc[i,j] ac[j] SEX[j] RACE[j] RCE[j] prevbr[j] BR1[j] BR2[j] BR3[j] BR4[j] BR5[j] 
     omsnbr[j];
quit;

proc sql;
  select * from solt order by rank;
quit;

data sol;
  set solt(rename=(BR=assigned arc=score));
  choice = '6other';
  if assigned = br1 then choice = '1st';
  else if assigned = br2 then choice = '2nd';
  else if assigned = br3 then choice = '3rd';
  else if assigned = br4 then choice = '4th';
  else if assigned = br5 then choice = '5th';
  _random = RANUNI(0);
  rnd = put(_random,10.8);
  rnk = put(rank,6.);
  obs = put(_N_,6.);
  scorec=put(score,10.8);
run;

proc sort data=sol;
by C;

ods csv file='./outo1/answer.csv';
proc print data=sol noobs;
run;
ods csv close;

proc sql;
  create table atemp1 as
  select rnk, count(*) as cnt
    from sol
    group by rnk
    order by rnk ascending;
quit;

proc sql;
  create table dups as
  select *
    from atemp1 
    where cnt > 1
    order by rnk;
quit;

proc sql;
  create table dups2 as
  select * 
    from sol
    where rnk in (select rnk from dups)
    order by rnk;
quit;

proc sql;
  create table maxes as
  select rnk, max(scorec||rnd) as mscore
    from dups2
    group by rnk;
quit;

proc sql;
  create table minimums as
  select rnk, min(scorec||rnd) as mscore
    from dups2
    group by rnk;
quit;

proc sql;
  title 'nodups';
  create table nodups as 
  select *
    from sol
    where rnk not in (select rnk from dups2);
  title;
quit;

proc sql;
  title 'dups to eliminate';
  create table dup_to_eliminate as 
  select *
    from sol
    where rnk in (select rnk from dups2);
  title;
quit;

proc sql;
  title 'resolved dups';
  create table dups_added_back as 
  select *
    from sol
    where rnk||scorec||rnd in (select rnk||mscore from maxes);
  title;
quit;

proc sql;
  create table answer2 as
  select * from nodups 
  union
  select * from dups_added_back;
quit;

proc sort data=answer2;
  by rank;

proc sql;
  create table quality as
  select assigned, avg(omsnbr) as avgoms
    from answer2
    group by assigned
    order avgoms desc;
quit;

proc sql;
  create table satisfy1 as
  select choice, count(*) as sat
    from answer2
	group by choice
	order by sat descending;
quit;

data sat1;
  set satisfy1;
  satstat=100*sat/&all_cnt;
run;

proc sort data=sat1;
  by choice;

data sat2;
  set sat1;
  retain cumstat;
  cumstat=sum(satstat,cumstat);
run;

proc sort data=sat2;
  by choice;

proc sql;
  create table sat3 as
  select choice, sat as count, satstat as satisfied, cumstat as cumulative
    from sat2
    order by choice;
quit;

proc sql;
  create table race1 as
  select assigned, count(*) as rce
    from answer2
	where rce='1'
	group by assigned
	order by assigned;
quit;
proc sql;
  create table rce4 as
  select assigned, count(*) as rce4
    from answer2
	where race='4'
	group by assigned
	order by assigned;
quit;
proc sql;
  create table rce3 as
  select assigned, count(*) as rce3
    from answer2
	where race='3'
	group by assigned
	order by assigned;
quit;
proc sql;
  create table rce2 as
  select assigned, count(*) as rce2
    from answer2
	where race='2'
	group by assigned
	order by assigned;
quit;
proc sql;
  create table rce5 as
  select assigned, count(*) as rce5
    from answer2
	where race='5'
	group by assigned
	order by assigned;
quit;

data rce1;
  set race1;
  rcestat=100*rce/&all_cnt;
run;

proc sql;
  title 'rce1';
  select * from rce1;
  title;
quit;

proc sql;
  create table females1 as
  select assigned, count(*) as fem
    from answer2
	where sex='F'
	group by assigned
	order by assigned;
quit;

data fem1;
  set females1;
  femstat=100*fem/(&all_cnt-&ca_cnt);
run;

proc sql;
  create table assigned1 as
  select assigned, count(*) as actasg
    from answer2
	group by assigned
	order by assigned;
quit;

data asg1;
  set assigned1;
  femstat=100*actasg/&all_cnt;
run;

proc sort data=asg1;
  by assigned;
proc sort data=fem1;
  by assigned;
proc sort data=rce1;
  by assigned;
proc sort data=rce4;
  by assigned;
proc sort data=rce3;
  by assigned;
proc sort data=rce2;
  by assigned;
proc sort data=rce5;
  by assigned;
proc sort data=goals;
  by BR;

/* "assigned","actasg","fem","rce","_sd_","femgoals","mingoals" */
data rcegen2(drop=femstat rcestat goalnbr);
  merge asg1 fem1 rce1 rce2 rce3 rce4 rce5 goals(rename=(BR=assigned));
  by assigned;
  if actasg = . then actasg =0;
  if fem = . then fem =0;
  if rce = . then rce =0;
  if rce2 = . then rce2 =0;
  if rce3 = . then rce3 =0;
  if rce4 = . then rce4 =0;
  if rce5 = . then rce5 =0;
  if rceT = . then rceT =0;
  asgdif=actasg-_sd_;
  femdif=fem-femgoals;
  rcedif=rce-mingoals;
  rceMdif=rceM-_sdmM_;
  rce2dif=rce2-min2goals;
  rce3dif=rce3-min3goals;
  rce4dif=rce4-min4goals;
  rce5dif=rce5-min5goals;
  rceTdif=rceT-minTgoals;
  rcedif2=rce-_sdm2_;
run;

proc sql;
  create table rcegen3 as
  select assigned as BBR, actasg as asgn,
         fem as fems, femgoals as fem_g, femdif as delta_f,
         rce as mins, mingoals as min_g, rcedif as delta_m,
         rce2 as mins2, min2goals as min2_g, rce2dif as delta_2m,
         rce3 as mins3, min3goals as min3_g, rce3dif as delta_3m,
         rce4 as mins4, min4goals as min4_g, rce4dif as delta_4m,
         rce5 as mins5, min5goals as min5_g, rce5dif as delta_5m
    from rcegen2
    order by assigned;
quit;

proc sql;
  create table rcegen3s as
  select assigned as BBR, actasg as asgn,
         fem as fems, femgoals as fem_g, femdif as delta_f,
         rce as mins, mingoals as min_g, rcedif as delta_m
    from rcegen2
    order by assigned;
quit;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\sat.csv'; */
/* outputs cadet satisfaction by choice */
ods csv file='./outo1/satisfied.csv';
proc print data=sat3 noobs;
run;
ods csv close;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\goals1.csv'; */
/* outputs goals data along with actual assignments and differences  */
ods csv file='./outo1/goals1.csv';
proc print data=rcegen3 noobs;
run;
ods csv close;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\goals2.csv'; */
/* outputs goals data along with actual assignments and differences  */
ods csv file='./outo1/goals2.csv';
proc print data=rcegen3s noobs;
run;
ods csv close;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\quality.csv'; */
/* outputs average branch quality  */
ods csv file='./outo1/quality.csv';
proc print data=quality noobs;
run;
ods csv close;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\dups.csv'; */
ods csv file='./outo1/dups.csv';
/* outputs duplicates details from the fractions created by the quality constraint  */
proc print data=dups2 noobs;
run;
ods csv close;

/* create Excel .csv file of the data requested    */
/* ods csv file='C:\Documents and Settings\mcg\Desktop\optm\outo1\goals.csv'; */

ods csv file='./outo1/goals.csv';
proc print data=goals noobs;
run;
ods csv close;
